/*-*- mode:c;indent-tabs-mode:nil;c-basic-offset:2;tab-width:8;coding:utf-8 -*-│
│ vi: set et ft=c ts=2 sts=2 sw=2 fenc=utf-8                               :vi │
╞══════════════════════════════════════════════════════════════════════════════╡
│ Copyright 2020 Justine Alexandra Roberts Tunney                              │
│                                                                              │
│ Permission to use, copy, modify, and/or distribute this software for         │
│ any purpose with or without fee is hereby granted, provided that the         │
│ above copyright notice and this permission notice appear in all copies.      │
│                                                                              │
│ THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL                │
│ WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED                │
│ WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE             │
│ AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL         │
│ DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR        │
│ PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER               │
│ TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR             │
│ PERFORMANCE OF THIS SOFTWARE.                                                │
╚─────────────────────────────────────────────────────────────────────────────*/
#include "libc/dce.h"
#include "libc/errno.h"
#include "libc/macros.h"
#include "libc/stdio/internal.h"
#include "libc/stdio/stdio.h"
#include "libc/str/str.h"

/**
 * Reads line from stream.
 *
 * This function is similar to getline() except it'll truncate lines
 * exceeding size. The line ending marker is included and may be removed
 * using chomp().
 *
 * @param s is output buffer
 * @param size is capacity of s
 * @param f is non-null file object stream pointer
 * @return s on success, NULL on error, or NULL if EOF happens when
 *     zero characters have been read
 */
char *fgets_unlocked(char *s, int size, FILE *f) {
  int c, n;
  char *p, *b, *t;
  p = s;
  if (size > 0) {
    while (--size > 0) {
      if (!IsTiny() && f->beg < f->end) {
        b = f->buf + f->beg;
        n = MIN(f->end - f->beg, size);
        if ((t = memchr(b, '\n', n))) {
          n = t + 1 - b;
        }
        if (n)
          memcpy(p, b, n);
        f->beg += n;
        size -= n - 1;
        p += n;
        if (t)
          break;
      } else {
        if ((c = fgetc_unlocked(f)) == -1) {
          break;
        }
        *p++ = c & 255;
        if (c == '\n')
          break;
      }
    }
    if (p > s || f->state != -1) {
      *p = '\0';
    }
  }
  return p > s ? s : NULL;
}
